/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package filesender;

import java.io.*;
import java.net.Socket;

/**
 * ファイル送受信などを実装したセッション抽象クラス
 * @author tondol
 */
public abstract class AbstractSession {
    protected Socket socket = null;
    protected BufferedReader reader = null;
    protected PrintStream writer = null;
    protected Thread thread = null;
    static final int BUF_SIZE = 65536;
    
    /**
     * セッションを実行
     */
    public abstract void run();
    
    /**
     * セッションを終了
     */
    public abstract void close();
    
    /**
     * メッセージ解釈
     * @param msg
     */
    public void executeMessage(String msg) {
        String cmd[] = msg.split("::", 2);
        if (cmd[0].equals("MSG")) {
            Main.window.putMessage(">" + cmd[1]);
        } else if (cmd[0].equals("SEND")) {
            Main.recvFile(cmd[1]);
        } else if (cmd[0].equals("RECV")) {
            Main.setSendReply(cmd[1]);
        }
    }
    
    /**
     * メッセージ送信
     * @param msg
     */
    public void sendMessage(String msg) {
        writer.println(msg);
        writer.flush();
    }
    
    /**
     * ファイル送信下請け
     * @param filename 送信ファイルのパス
     * @param size 送信ファイルのサイズ
     * @throws java.lang.IOException
     */
    public void sendFile(String filename, long size) throws IOException {
        BufferedInputStream in = null;
        BufferedOutputStream out = null;
        try {
            // ストリーム等の確保
            in = new BufferedInputStream(new FileInputStream(filename));
            out = new BufferedOutputStream(socket.getOutputStream());
            byte buf[] = new byte[BUF_SIZE];
            long remain = size;
            // ファイルの送信
            while (remain > 0) {
                int readSize = in.read(buf, 0, BUF_SIZE);
                // 読み込みに失敗したら脱出
                if (readSize == -1) throw new IOException();
                out.write(buf, 0, readSize);
                out.flush();
                remain -= readSize;
                // 進捗を更新
                int per = (int)(100.0 - (double)remain / (double)size * 100.0);
                Main.window.changeProgress(per);
            }
        } catch(IOException e) {
            throw e;
        } finally {
            in.close();
            //out.close();
        }
    }
    
    /**
     * ファイル受信下請け
     * @param filename 受信ファイルの保存先
     * @param size 受信ファイルのサイズ
     * @throws java.lang.IOException
     */
    public void recvFile(String filename, long size) throws IOException {
        BufferedInputStream in = null;
        BufferedOutputStream out = null;
        try {
            // ストリーム等の確保
            in = new BufferedInputStream(socket.getInputStream());
            out = new BufferedOutputStream(new FileOutputStream(filename));
            long remain = size;
            byte buf[] = new byte[BUF_SIZE];
            // ファイルの受信
            while (remain > 0) {
                int readSize = in.read(buf, 0, BUF_SIZE);
                // 読み込みに失敗したら脱出
                if (readSize == -1) throw new IOException();
                out.write(buf, 0, readSize);
                remain -= readSize;
                // 進捗を更新
                int per = (int)(100.0 - (double)remain / (double)size * 100.0);
                Main.window.changeProgress(per);
            }
        } catch(IOException e) {
            throw e;
        } finally {
            //in.close();
            out.close();
        }
    }
}
