/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package filesender;

import java.io.File;
import java.io.IOException;

/**
 * アプリ管理クラス
 * @author tondol
 */
public class Main {
    // 管理するクラスとか
    static MainView window = null;
    static AbstractSession session = null;
    static Config config = null;
    private static String sendReply = null;
    private static boolean isLocked = false;
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        window = MainView.getInstance();
        config = new Config();
    }
    
    /**
     * アプリケーションを終了
     */
    public static void exit() {
        config.save();
    }
    
    /**
     * サーバーセッションを開始
     */
    public static void runServer() {
        try {
            window.setState(MainView.BUSY);
            window.putMessage("+starts server");
            session = new ServerSession(config.listen);
        } catch(IOException e) {
            window.setState(MainView.CLOSED);
            window.putMessage("-starting failed");
        }
    }
    
    /**
     * クライアントセッションを開始
     */
    public static void runClient() {
        try {
            window.setState(MainView.BUSY);
            window.putMessage("+starts client");
            session = new ClientSession(config.host, config.port);
            window.setState(MainView.CONNECTED);
            window.putMessage("+client accepted");
        } catch (IOException e) {
            window.setState(MainView.CLOSED);
            window.putMessage("-starting failed");
        }
    }
    
    /**
     * セッションを終了
     */
    public static void closeSession() {
        session.close();
        session = null;
    }
    
    /**
     * 相手にファイルを送信
     * @param file 送信ファイルの構造体
     */
    public static void sendFile(File file) {
        // 状態をロック
        lock();
        // ファイル情報を取得
        String filename = file.getAbsolutePath();
        String lastname = file.getName();
        long size = file.length();
        // 送信確認を送信
        session.sendMessage("SEND::" + lastname + "::" + size);
        window.setState(MainView.BUSY);
        window.putMessage("+sending <" + lastname + ">");
        String reply = getSendReply();
        // 返信をチェック
        if (reply.equals("NG")) {
            window.setState(MainView.CONNECTED);
            window.putMessage("+denied <" + lastname + ">");
            setSendReply(null);
            unlock();
            return;
        } else {
            window.putMessage("+accepted <" + lastname + ">");
        }
        // 送信開始
        try {
            session.sendFile(filename, size);
            window.putMessage("+complete <" + lastname + ">");
        } catch(IOException e) {
            window.putMessage("-failed <" + lastname + ">");
        } finally {
            window.setState(MainView.CONNECTED);
            setSendReply(null);
            unlock();
        }
    }
    
    /**
     * 相手からファイルを受信
     * @param msg RECVコマンドの引数
     */
    public static void recvFile(String msg) {
        // 状態をロック
        lock();
        // ファイル情報を取得
        String cmd[] = msg.split("::", 2);
        String filename = cmd[0];
        long size = Long.parseLong(cmd[1]);
        String info = filename + " (" + size + "bytes)";
        // ファイル選択ダイアログを開く
        RecvDialog dialog = new RecvDialog(window, true, info);
        dialog.setVisible(true);
        // 受信可否を送信
        if (dialog.isAccept()) {
            window.setState(MainView.BUSY);
            window.putMessage("+receiving <" + filename + ">");
            session.sendMessage("RECV::OK");
        } else {
            session.sendMessage("RECV::NG");
            unlock();
            return;
        }
        // 受信開始
        try {
            session.recvFile(config.directory + filename, size);
            window.putMessage("+complete <" + filename + ">");
        } catch(IOException e) {
            window.putMessage("-failed <" + filename + ">");
        } finally {
            window.setState(MainView.CONNECTED);
            unlock();
        }
    }
    
    /**
     * 自前のロックを設定する
     */
    private static void lock() {
        while (isLocked == true) { }
        isLocked = true;
    }
    
    /**
     * 自前のロックを解除する
     */
    private static void unlock() {
        isLocked = false;
    }
    
    /**
     * 送信確認の返答を取得
     * @return
     */
    private static String getSendReply() {
        while (sendReply == null) {
            try { Thread.sleep(100); } catch(InterruptedException e) { }
        }
        return sendReply;
    }
    
    /**
     * 送信確認の返答を設定
     * @param reply
     */
    public static void setSendReply(String reply) {
        sendReply = reply;
    }
}
